// Copyright (c) 2014 Ashley Jeffs
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

// Package types defines any general structs and interfaces used throughout the
// benthos code base.
//
// Benthos uses abstract types to represent arbitrary producers and consumers of
// data to its core components. This allows us to construct types for piping
// data in various arrangements without regard for the specific destinations and
// sources of our data.
//
// The basic principle behind a producer/consumer relationship is that a
// producer pipes data to the consumer in lock-step, where for each message sent
// it will expect a response that confirms the message was received and
// propagated onwards.
//
// Messages and responses are sent via channels, and in order to instigate this
// pairing each type is expected to create and maintain ownership of its
// respective sending channel.
package types
